//
// ChangePhotoPath.IChangePhotoPathController.cs: The logic to change the photo path in photos.db
//
// Author:
//   Bengt Thuree (bengt@thuree.com)
//
// Copyright (C) 2007
// 


using FSpot;
using System;
using System.IO;
//using Mono.Unix; Do not know where to find the .pc file for this one. 
#if ENABLE_NUNIT
using NUnit.Framework;
#endif

/*
	Need to
		1) Find old base path, assuming starting /YYYY/MM/DD so look for /YY (/19 or /20)
		2) Confirm old base path and display new base path
		3) change every instance of old base path to new path

Consider!!!
photo_store.Commit(photo) is using db.ExecuteNonQuery, which is not waiting for the command to finish. On my test set of 20.000 photos,
it took SQLite another 1 hour or so to commit all rows after this extension had finished its execution.


*/

namespace ChangePhotoPath
{
	public class ChangePathController
	{		
		PhotoStore photo_store = FSpot.Core.Database.Photos;	
		int total_photos;
		string orig_base_path;

		private const string BASE2000 = "/20";
		private const string BASE1900 = "/19";
	
		private IChangePhotoPathGui gui_controller;

		public ChangePathController (IChangePhotoPathGui gui)
		{
			gui_controller = gui;	
			total_photos = photo_store.TotalPhotos;
			orig_base_path = FindOrigBasePath();
			string new_base_path = FSpot.Global.PhotoDirectory;
			gui_controller.DisplayDefaultPaths (orig_base_path, new_base_path);
			gui_controller.UpdateProgressBar (total_photos);
		}

		public bool CanWeRun ()
		{
			return (orig_base_path != null);
		}

		public void ChangeAllPhotosBasePath (string old_base, string new_base)
		{
			old_base = EnsureEndsWithOneDirectorySeparator (old_base);
			new_base = EnsureEndsWithOneDirectorySeparator (new_base);
			int count = 0;
			foreach ( IBrowsableItem ibrows in photo_store.Query ( (Tag [])null, (Roll [])null) ) {
				count++;
				gui_controller.UpdateProgressBar (count, total_photos);
				string tmp_path = (ibrows as Photo).DefaultVersionUri.AbsolutePath;
				if ( tmp_path.StartsWith (old_base) )
					ChangeThisPhoto (ibrows as Photo, old_base, new_base);
				else 
					System.Console.WriteLine ("Skipped this file. Linked...{0}", tmp_path);
			}

			System.Console.WriteLine ("Total photos processed...{0} out of {1}", count, total_photos);

		}
	
		private string EnsureEndsWithOneDirectorySeparator (string tmp_str)
		{
			while (tmp_str.EndsWith(String.Format ("{0}", Path.DirectorySeparatorChar)))
				tmp_str = tmp_str.Remove (tmp_str.Length-1, 1);
			return String.Format ("{0}{1}", tmp_str, Path.DirectorySeparatorChar);
		}

		private void ChangeThisPhoto (Photo photo, string old_base, string new_base)
		{
			string path = string.Format ("{0}{1}", new_base, photo.DefaultVersionUri.AbsolutePath.Substring(old_base.Length));
System.Console.WriteLine ("old uri = {0}, new uri {1}", photo.DefaultVersionUri.AbsolutePath, path);
System.Console.Write (".");
			photo.UpdateUri ( new System.Uri ( EnsureEndsWithOneDirectorySeparator(path)) );
			photo_store.Commit (photo);
		}

		private string IsThisPhotoOnOrigBasePath (string check_this_path)
		{
			int i;
			i = check_this_path.IndexOf(BASE2000);
			if (i > 0) 
				return (check_this_path.Substring(0, i));
			i = check_this_path.IndexOf(BASE1900);			
			if (i > 0) 
				return (check_this_path.Substring(0, i));
			return null;
		}

		private string FindOrigBasePath()
		{
			string res_path = null;
			foreach ( IBrowsableItem photo in photo_store.Query ( (Tag [])null, (Roll [])null) ) {
				string tmp_path = (photo as Photo).DefaultVersionUri.AbsolutePath;
				res_path = IsThisPhotoOnOrigBasePath (tmp_path);
				if (res_path != null)
					break;			
			}
			return res_path;
		}
#if ENABLE_NUNIT
                [TestFixture]
                public class Tests 
                {
                        [Test]
                        public void TestPng ()
			{
                        }

                        public void Basic (string name)
                        {
                                string path = CreateFile (name, 120);
                                using (FilterRequest req = new FilterRequest (path)) {
                                        IFilter filter = new TiltFilter (Math.PI / 4);
                                        filter.Convert (req);
                                        Assert.IsTrue (System.IO.File.Exists (req.Current.LocalPath),
                                                       "Error: Did not create " + req.Current.LocalPath);
                                        Assert.IsTrue (new FileInfo (req.Current.LocalPath).Length > 0,
                                                       "Error: " + req.Current.LocalPath + "is Zero length");
                                }
                        }
                }
#endif

	}
}

