//
// ChangePhotoPath.IChangePhotoPathGui.cs: The Gui to change the photo path in photos.db
//
// Author:
//   Bengt Thuree (bengt@thuree.com)
//
// Copyright (C) 2007
// 

using FSpot.Extensions;
using System;
using Gnome.Vfs;
//using Mono.Unix; Do not know where to find the .pc file for this one. 

namespace ChangePhotoPath
{

	public class Dump : ICommand, IChangePhotoPathGui
	{
		private string dialog_name = "ChangePhotoPath";
		private Glade.XML xml;
		private Gtk.Dialog dialog;
		private ChangePathController contr;

		[Glade.Widget] Gtk.Entry old_common_uri;
		[Glade.Widget] Gtk.Label new_common_uri;
		[Glade.Widget] Gtk.ProgressBar progress_bar;

		private bool LaunchController()
		{		
			try {
				contr = new ChangePathController ( this );
			} catch (Exception e) {
				Console.WriteLine(e);
				return false;
			}

			return true;	
		}

		public void LaunchDialog()
		{
			CreateDialog();
			Dialog.Modal = false;
			Dialog.TransientFor = null;
			if (LaunchController() && contr.CanWeRun())
			{			
				Dialog.ShowAll();
				Dialog.Response += HandleResponse;
			} else {
				DisplayCanNotRunMsg();
				Dialog.Destroy();
			}
		}

		private void CreateDialog()
		{
			xml = new Glade.XML (null, "ChangePhotoPath.glade", dialog_name, "f-spot");
			xml.Autoconnect (this);
		}

		private Gtk.Dialog Dialog {
			get {
				if (dialog == null)
					dialog = (Gtk.Dialog) xml.GetWidget (dialog_name);
				return dialog;
			}
		}
				 
		private void HandleResponse (object sender, Gtk.ResponseArgs args)
		{
			if (args.ResponseId == Gtk.ResponseType.Ok)
				contr.ChangeAllPhotosBasePath(old_common_uri.Text, new_common_uri.Text);
			Dialog.Destroy();

			return;
		}

		public void DisplayDefaultPaths (string oldpath, string newpath)
		{
			old_common_uri.Text = oldpath;
			new_common_uri.Text = newpath;
		}

		private void DisplayCanNotRunMsg()
		{
			HigMessageDialog.RunHigMessageDialog (	null,  
								Gtk.DialogFlags.Modal | Gtk.DialogFlags.DestroyWithParent,
								Gtk.MessageType.Error, 
								Gtk.ButtonsType.Ok,
// Do not know where to find the library for this Catalog method
// Catalog.GetString ("Only linked photos, not possible to continue."), 
						      		"Only linked photos, not possible to continue.", 
								null);
		}

		public void UpdateProgressBar (int count, int total_photos)
		{
			if (progress_bar != null)
			{
				progress_bar.Text = (String.Format ("Processing {0} of {1}", count, total_photos));
				progress_bar.Fraction = (double) count / System.Math.Max (total_photos, 1);
			}
		}

		public void UpdateProgressBar (int total_photos)
		{
			if (progress_bar != null)
			{
				progress_bar.Text = (String.Format ("{0} photos to check", total_photos));
			}
		}
	
		public void Run (object sender, EventArgs args)
		{
			try {
				LaunchDialog( );
			} catch (Exception e) {
				Console.WriteLine(e);
			}
		}
	}
}
